/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIX509Cert.idl"

interface nsIArray;
interface nsIASN1Object;

%{ C++
 /* forward declaration */
 typedef struct CERTCertificateStr CERTCertificate;
%}
[ptr] native CERTCertificatePtr(CERTCertificate);

/**
 * This represents additional interfaces to X.509 certificates
 */
[scriptable, uuid(5b62c61c-f898-4dab-8ace-51109bb459b4)]
interface nsIX509Cert2 : nsIX509Cert {
  /**
   *  Additional constants to classify the type of a certificate.
   */
  const unsigned long ANY_CERT  = 0xffff;
  readonly attribute unsigned long certType;
  void markForPermDeletion();
  [notxpcom, noscript] CERTCertificatePtr getCert();
};
