/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_WorkerConsoleBinding_h__
#define mozilla_dom_WorkerConsoleBinding_h__

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

class NativePropertyHooks;
class ProtoAndIfaceArray;
struct WorkerConsoleStackAtoms;

namespace workers {

class WorkerConsole;

} // namespace workers

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct WorkerConsoleStack : public MainThreadDictionaryBase
{
  nsString mFilename;
  nsString mFunctionName;
  uint32_t mLanguage;
  uint32_t mLineNumber;

  WorkerConsoleStack();

  explicit inline WorkerConsoleStack(const WorkerConsoleStack& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline WorkerConsoleStack(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, WorkerConsoleStackAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const WorkerConsoleStack& aOther);
};

namespace binding_detail {
struct FastWorkerConsoleStack : public WorkerConsoleStack
{
  inline FastWorkerConsoleStack()
    : WorkerConsoleStack(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


namespace WorkerConsoleBinding_workers {

  typedef mozilla::dom::workers::WorkerConsole NativeType;

  const JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, mozilla::dom::workers::WorkerConsole* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, T* aObject)
  {
    return Wrap(aCx, aScope, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceArray& aProtoAndIfaceArray, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace WorkerConsoleBinding_workers



} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_WorkerConsoleBinding_h__
